package yrpg.bamp.basedata.plugin.operate;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.bos.servicehelper.org.OrgUnitServiceHelper;
import kd.sdk.plugin.Plugin;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 单据操作插件同步报销级别
 */
public class ReimbursementLevelSynchronousOpPlugin extends AbstractOperationServicePlugIn implements Plugin {
    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        e.getFieldKeys().add("number");
        e.getFieldKeys().add("yrpg_rank");
        e.getFieldKeys().add("entryentity.dpt");

    }

    @Override
    public void beforeExecuteOperationTransaction(BeforeOperationArgs e) {
        super.beforeExecuteOperationTransaction(e);
        List<DynamicObject> saveBills = new ArrayList<>();
        List<DynamicObject> updateBills = new ArrayList<>();

        for (DynamicObject dataEntity : e.getDataEntities()) {
            String rank = dataEntity.getString("yrpg_rank");
            String number = dataEntity.getString("number");
            if (!rank.isEmpty()){
                DynamicObjectCollection entryentity = dataEntity.getDynamicObjectCollection("entryentity");
                List<Long> orgIds = new ArrayList<>();
                for (DynamicObject row : entryentity) {
                    long dptId=row.getDynamicObject("dpt").getLong("id");
                    Map<String, Object> companyInfoMap = OrgUnitServiceHelper.getCompanyfromOrg(dptId);
                    if (!companyInfoMap.isEmpty()){
                        if (!orgIds.contains((long)companyInfoMap.get("id"))){
                            orgIds.add((long)companyInfoMap.get("id"));
                        }

                    }

                }
                QFilter qFilter = new QFilter("number", QCP.equals, rank);
                DynamicObject levelBill = BusinessDataServiceHelper.loadSingle("yrpg_reimbursement", new QFilter[]{qFilter});
                if (levelBill!=null){
                    DynamicObject level = levelBill.getDynamicObject("yrpg_level");
                    QFilter qFilter1 = new QFilter("user.number", QCP.equals, number);
                    QFilter qFilter2 = new QFilter("company.id", QCP.in, orgIds);
                    DynamicObject[] bills = BusinessDataServiceHelper.load("er_reimbursesetting_rel", "reimburselevel,company", new QFilter[]{qFilter1,qFilter2});
                    for (DynamicObject bill : bills) {
                        bill.set("reimburselevel",level);
                        orgIds.remove(bill.getDynamicObject("company").getLong("id"));
                        updateBills.add(bill);
                    }


                    for (Long orgId : orgIds) {
                        DynamicObject newBill = BusinessDataServiceHelper.newDynamicObject("er_reimbursesetting_rel");
                        newBill.set("user",dataEntity);
                        newBill.set("reimburselevel",level);
                        newBill.set("company",orgId);
                        saveBills.add(newBill);
                    }
                }
            }

        }
        DynamicObject[] saveBill = saveBills.toArray(new DynamicObject[0]);
        DynamicObject[] updateBill = updateBills.toArray(new DynamicObject[0]);
        SaveServiceHelper.save(saveBill);
        SaveServiceHelper.update(updateBill);
    }
}